/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.plugin;

import com.google.common.collect.ImmutableCollection;
import com.google.common.collect.ImmutableList;
import com.google.inject.Injector;
import com.sos.scheduler.engine.eventbus.EventBus;
import com.sos.scheduler.engine.kernel.Scheduler;
import com.sos.scheduler.engine.kernel.command.CommandHandler;
import com.sos.scheduler.engine.kernel.scheduler.SchedulerException;
import com.sos.scheduler.engine.kernel.scheduler.Subsystem;
import com.sos.scheduler.engine.kernel.command.HasCommandHandlers;
import com.sos.scheduler.engine.kernel.log.PrefixLog;
import com.sos.scheduler.engine.kernel.util.Lazy;

import java.util.HashMap;
import java.util.Map;
import org.w3c.dom.Element;

import static com.google.common.base.Throwables.getStackTraceAsString;
import static com.sos.scheduler.engine.kernel.util.XmlUtils.*;

public final class PluginSubsystem implements Subsystem, HasCommandHandlers {
    private final CommandHandler[] commandHandlers = {
            new PluginCommandExecutor(this),
            new PluginCommandCommandXmlParser(this),
            new PluginCommandResultXmlizer(this) };
    private final Lazy<Injector> lazyInjector;
    private final Map<String,PluginAdapter> plugins = new HashMap<String,PluginAdapter>();
    private final EventBus eventBus;
    private final PrefixLog log;

    public PluginSubsystem(Scheduler scheduler, Lazy<Injector> injector, EventBus eventBus) {
        this.lazyInjector = injector;
        this.eventBus = eventBus;
        this.log = scheduler.log();
    }

    public void load(Element root) {
        PluginReader pluginReader = new PluginReader(log);
        Element pluginsElement = elementXPathOrNull(root, "config/plugins");
        if (pluginsElement != null) {
            for (Element e: elementsXPath(pluginsElement, "plugin")) {
                PluginAdapter a = pluginReader.tryReadPlugin(e);
                if (a != null) {
                    plugins.put(a.getPluginClassName(), a);
                    log.info(a + " added");
                }
            }
        }
    }

    public void activate() {
        for (PluginAdapter p: plugins.values())
            if (p.getConfiguration().getActivationMode() == ActivationMode.activateOnStart)
                activatePluginAdapter(p);
    }

    public void activatePlugin(Class<? extends Plugin> c) {
        activatePluginAdapter(pluginAdapterByClassName(c.getName()));
    }

    void activatePluginAdapter(PluginAdapter p) {
        p.tryActivate(lazyInjector.get());
        eventBus.registerAnnotated(p.getPlugin());
    }

    public void close() {
        for (PluginAdapter p: plugins.values()) {
            eventBus.unregisterAnnotated(p.getPlugin());
            p.tryClose();
        }
    }

    PluginAdapter pluginAdapterByClassName(String className) {
        PluginAdapter result = plugins.get(className);
        if (result == null)
            throw new SchedulerException("Unknown plugin '" + className + "'");
        return result;
    }

    @SuppressWarnings("unchecked")
    public <T extends Plugin> T pluginByClass(Class<T> c) {
        return (T)pluginAdapterByClassName(c.getName()).getPlugin();
    }

    @Override public ImmutableCollection<CommandHandler> getCommandHandlers() {
        return ImmutableList.copyOf(commandHandlers);
    }

    static void logError(PrefixLog log, String pluginName, Throwable t) {
        log.error(pluginName +": "+ t +"\n"+ getStackTraceAsString(t));
    }
}
